module Pages.Services exposing (Model, Msg, page)

import Browser
import Config.Data.Identity exposing (pageNames)
import Config.Helpers.CardFormat exposing (cardMaker)
import Config.Helpers.Format
    exposing
        ( headerFontSizeMedium
        , headerFontSizeSmall
        , paragraphFontSize
        , paragraphSpacing
        )
import Config.Helpers.Headers.Header exposing (headerMaker)
import Config.Helpers.Headers.Types exposing (Header)
import Config.Helpers.Response
    exposing
        ( pageList
        , topLevelContainer
        )
import Config.Helpers.Viewport exposing (resetViewport)
import Config.Pages.Services.Records.DebateAnalysis exposing (..)
import Config.Pages.Services.Records.DebateCoaching exposing (..)
import Config.Pages.Services.Records.ElmBuilds exposing (..)
import Config.Pages.Services.Records.NixBuilds exposing (..)
import Config.Pages.Services.Records.NutritionScience exposing (..)
import Config.Pages.Services.Types exposing (..)
import Config.Style.Colour as T exposing (..)
import Config.Style.Glow exposing (glowDeepDarkGrey, glowDeepDarkOrange)
import Config.Style.Transitions exposing (transitionStyleMedium)
import Effect exposing (Effect)
import Element as E exposing (..)
import Element.Background as B
import Element.Border as D
import Element.Font as F
import Layouts
import Page exposing (Page)
import Route exposing (Route)
import Shared exposing (..)
import View exposing (View)


page : Shared.Model -> Route () -> Page Model Msg
page shared route =
    Page.new
        { init = init
        , update = update
        , subscriptions = subscriptions
        , view = view shared
        }
        |> Page.withLayout toLayout


toLayout : Model -> Layouts.Layout Msg
toLayout model =
    Layouts.Navbar {}



-- INIT


type alias Model =
    {}


init : () -> ( Model, Effect Msg )
init () =
    ( {}
    , Effect.map
        (\_ -> NoOp)
        (Effect.sendCmd resetViewport)
    )



-- UPDATE


type Msg
    = NoOp


update : Msg -> Model -> ( Model, Effect Msg )
update msg model =
    case msg of
        NoOp ->
            ( model
            , Effect.none
            )



-- SUBSCRIPTIONS


subscriptions : Model -> Sub Msg
subscriptions model =
    Sub.none



-- VIEW


view : Shared.Model -> Model -> View Msg
view shared model =
    { title = pageNames.pageServices
    , attributes = []
    , element = servicesContainer shared.device
    }


servicesContainer : Device -> Element msg
servicesContainer device =
    topLevelContainer (servicesList device)


servicesList : Device -> Element msg
servicesList device =
    column
        (case ( device.class, device.orientation ) of
            _ ->
                pageList
        )
    <|
        List.concat
            [ List.map (headerMaker device)
                [ servicesHeader ]
            , (case ( device.class, device.orientation ) of
                _ ->
                    List.map (serviceMaker device)
              )
                [ servicesDebateAnalysis
                , servicesDebateCoaching
                , servicesNutritionScience
                , servicesNixBuilds
                , servicesElmBuilds
                ]
            ]


servicesHeader : Header
servicesHeader =
    let
        name =
            "Services"
    in
    { headerTitle = String.toUpper name
    , headerBody = "Bundle any of the services below to receive a $10 discount per hour. For example, two Debate Analysis sessions would cost $140 total, instead of $80/hr. All prices are in CAD."
    }


serviceMaker : Device -> Service msg -> Element msg
serviceMaker device service =
    cardMaker device service.serviceName (contentList service) (serviceImage service) service.serviceLink


contentList : Service msg -> List (Element msg)
contentList service =
    [ rateMaker service
    , descriptionMaker service
    , offeringMaker service
    ]


serviceImage : Service msg -> { src : String, description : String }
serviceImage service =
    { src = "services/" ++ service.serviceImage ++ ".png"
    , description = service.serviceName
    }


serviceWidth =
    width <| px 45


rateMaker : Service msg -> Element msg
rateMaker service =
    row
        ([ F.color colourTheme.textLightOrange
         , paragraphSpacing
         , paragraphFontSize
         , F.bold
         ]
            ++ [ F.size 18
               , E.width fill
               ]
        )
        [ column
            [ alignTop
            , serviceWidth
            ]
            [ E.text "Rate:"
            ]
        , column
            [ E.width fill
            , alignLeft
            ]
            [ el
                [ F.regular
                , paragraphFontSize
                , F.color colourTheme.textLightGrey
                ]
              <|
                E.text service.serviceRate
            ]
        ]


descriptionMaker : Service msg -> Element msg
descriptionMaker service =
    row
        [ F.color colourTheme.textLightOrange
        , paragraphSpacing
        , headerFontSizeSmall
        , F.bold
        ]
        [ column
            [ alignTop
            , width <| px 80
            ]
            [ E.text "Offerings:"
            ]
        ]


offeringMaker : Service msg -> Element msg
offeringMaker service =
    column
        [ spacing 8
        , width fill
        , paddingEach
            { top = 0
            , right = 0
            , bottom = 0
            , left = 35
            }
        ]
    <|
        List.map2 (\x y -> makeDescription x)
            service.serviceDescription
            (List.range 1 (List.length service.serviceDescription))


makeDescription : Description -> Element msg
makeDescription description =
    column
        ([ F.color colourTheme.textLightGrey
         , paragraphSpacing
         , paragraphFontSize
         , F.bold
         , alignLeft
         ]
            ++ [ spacing 8
               , width fill
               ]
        )
        [ paragraph [ F.regular ]
            [ E.text ("‣ " ++ description.point) ]
        ]
